%{ 
RE ANDREWS AND BARWICK (2012) PAPER:
THIS PROCEDURE IS USED BY THE PROGRAM rmsprg_fs_short_final.m AND COMPUTES CRITICAL VALUES OF TYPE:
      CVTYPE == 0   STD PLUG-IN ASYMPTOTIC, I.E., PA (UNCONDITIONAL),
      CVTYPE == 1   PHI1, t TEST, (A CONDITIONAL CV),
      CVTYPE == 11 PHI1, t TEST, WITH AUTOMATIC DETERMINATION OF KAPPA and ETA BASED
                                  ON V, 

IF A BOOTSTRAP RMS CRITICAL VALUE IS DESIRED, ONE ADDS 10 BEFORE THE NUMBER OF THE CVTYPE.  
THUS,
      CVTYPE == 100    BOOTSTRAP PLUG-IN ASY CRIT VAL,
      CVTYPE == 101    BOOTSTRAP PHI1, t TEST, (A CONDITIONAL CV),
      CVTYPE == 1011  BOOTSTRAP PHI1, t TEST, WITH AUTOMATIC DETERMINATION OF KAPPA 
                                       and ETA BASED ON V, 

THE PROCEDURE TAKES A P BY 1 VECTOR OF DATA, W, and COMPUTES A CONDITIONAL CRITICAL 
VALUE (I.E., QUANTILE) OF LEVEL 1-SIGLEVEL OF A TEST STATISTIC GIVEN W.
THE DATA ARE GENERATED WITH COVARIANCE MATRIX V and MEAN ZERO.  
THE TYPE OF TEST STATISTIC IS SPECIFIED BY STATTYPE.  IT CAN BE A MMM/SUM (S_1)
OR AQLR (S_2A) TEST STATISTIC.

WHEN A BOOTSTRAP CRIT VAL IS TO BE COMPUTED, THE MATRIX XMAT THAT IS 
THE THIRD ARGUMENT OF THE PROCEDURE NEEDS TO BE THE N BY P MATRIX OF DATA.  (NOTE THAT
IN THIS CASE, W EQUALS N^(1/2) AVG(XMAT).)  IF A BOOTSTRAP CRIT VAL IS NOT TO BE COMPUTED, 
XMAT CAN BE  ANYTHING.

IF CVTYPE = 1, THE CRIT VAL DEPENDS ON A POSITIVE SCALAR KAPPA, WHICH INDEXES HOW CLOSE 
THE CRIT VAL IS TO THE UNCONDITIONAL PA CRIT VAL. KAPPA = INFINITY YIELDS THE PA CRIT 
VAL.     

IF CVTYPE = 1, THE CRIT VAL DEPENDS ON A REAL NUMBER ETA, WHICH IS A SIZE-ADJUSTMENT 
FACTOR. THIS PROGRAM CAN BE USED TO CALCULATE THE APPROPRIATE ETA for A GIVEN TEST. WHEN
THE PROGRAM IS USED TO CALCULATE ETA, THE VALUE OF ETA THAT IS USED IN THIS PROCEDURE 
IS 0.  WHEN ETA IS GIVEN, THIS PROCEDURE JUSTS ADDS THIS VALUE TO THE CALCULATED 
CONDITIONAL CRITICAL VALUE.

IF CVTYPE = 11, THE CRIT VAL IS A t TEST CRIT VAL WITH KAPPA and ETA DETERMINED BY THE 
VARIANCE MATRIX V THROUGH THE QUANTITY DELTA(OMEGA), DENOTED BY DELTA IN THE PROGRAM, 
WHERE OMEGA IS THE CORRELATION MATRIX THAT CORRESPONDS TO V.  BY DEFINITION, 
DELTA(OMEGA) IS THE MIN OVER SUB-MATRICES OF C'OMEGA*C/|C|, WHERE C IS A P-VECTOR OF 
0'S and 1'S WITH AT LEAST TWO 1'S. DELTA(OMEGA) MEASURES THE AMOUNT OF CORRELATION IN 
OMEGA and DETERMINES HOW LARGE KAPPA NEEDS TO BE IN ORDER for THE TEST TO HAVE CORRECT 
REJECTION PROBABILITY WHEN SIGLEVEL IS .05 and ETA IS A SPECIFIED QUANTITY.

THE QUANTILE IS SIMULATED USING RANDOM NUMBERS WITH BLOCKS OF LENGTH R1 and
NUMBER OF BLOCKS GIVEN BY NUMR1.  HENCE THE TOTAL NUMBER OF RANDOM VARIABLES USED
IN THE SIMULATION IS NUMR1 * R1.
%}

function[ QUANTIL ]=CRITVALS_FS( W, V, XMAT, ETA, STATTYPE, CVTYPE, KAPPA, SIGLEVEL, R1, NUMR1)
    LONGVEC = zeros( NUMR1*R1, 1);
    N = size( XMAT, 1 );
    P = size( V, 1 );
    ETA1 = ETA; 
    SEEDQ =  942007;  % HERE WE USE THE DIFFERENT SEED FROM ORIGINAL PROGRAM.
    STREAMQ = RandStream.create('mt19937ar','seed',SEEDQ);
    RandStream.setDefaultStream(STREAMQ);
    

    WC = W./sqrt(diag(V));
         % WC IS THE RESCALED DATA VECTOR FOR WHICH EACH ELEMENT HAS VARIANCE
         % ONE, DENOTED WC.
    STACK = sqrt(diag(V));
    VC = V ./ (STACK * STACK');
         % VC IS THE CORRELATION MATRIX THAT CORRESPONDS TO V. 
    DELTA = min( VC(:) );
         % DELTA IS USED BY THE RECOMMENDED TEST, WHICH USES CVTYPE=11. IT MEASURES
         % THE RELEVANT AMOUNT OF DEPENDENCE IN THE V MATRIX. DELTA IS THE MINIMUM
         % OVER ALL ELEMENTS OF VC. 
    VCSQRT=sqrtm(VC); % V CANT BE SINGULAR

    if CVTYPE > 99;
       BOOT = 1; 	% ALL CVTYPES BIGGER THAN 99 CORRESPOND TO RMS BOOTSTRAP
                    % CRIT VALS. 
       NUMR1 = NUMR1*R1;
       R1 = 1;
              % IF BOOTSTRAP CRIT VALS ARE TO BE COMPUTED, WE HAVE TO DO THEM ONE 
              % AT A TIME, WE CANNOT DO THEM IN BLOCKS OF SIZE R1. SO WE RE-DEFINE
              % NUMR1 and R1 HERE SUCH THAT R1 = 1. 
    else
       BOOT = 0;
    end; 

    S = 1;
    while S <= NUMR1;
       STACK = sqrt(diag(V)); 
       VC = V ./ (STACK * STACK');  
                % HERE WE INITIALIZE VC AND VCINV. 
       VCINV = pinv( VC );
       VST = eye( P );  % HERE WE INITIALIZE VST AND VINVST, WHICH ARE USED WHEN
                        % BOOTSTRAP CRIT VALS ARE COMPUTED. 
       VCINVST = eye( P );
       if BOOT == 1;  	% IF BOOT EQUALS 1, WE CALCULATE ZQUANT USING THE
						% NONPARAMETRIC BOOTSTRAP.                           
          INDEX = floor( rand(N,1)*N +1); 	% THESE TWO LINES OF CODE
          XMATST = XMAT( INDEX, : );        % COMPUTE A BOOTSTRAP SAMPLE.  
          VST = cov( XMATST );  % VST IS THE BOOTSTRAP ESTIMATOR OF V. 
          STACK = sqrt(diag(VST));
          VCST = VST ./ (STACK * STACK'); 
                % VCST IS THE BOOTSTRAP ESTIMATOR OF THE BOOTSTRAP 
                % CORRELATION MATRIX VC.  
          ZQUANT = sqrt(N)*( mean(XMATST) - mean(XMAT) )./sqrt(diag(VST))';   
                %  ZQUANT IS A 1 BY P VECTOR WHEN BOOT = 1.  
       elseif BOOT == 0;  
                % IF BOOT DOES NOT EQUAL 1, WE CALCULATE ZQUANT USING THE
                %   ASY DISTRIBUTION.      
          ZQUANT = randn( R1, P )*VCSQRT;    
                % NOTE THAT VC IS THE CORRELATION MATRIX BASED ON V.  
       end;

       if CVTYPE == 0||CVTYPE == 100;   
                % CVTYPE == 0 IS THE STD PLUG-IN ASY (PA) CRIT VALUE.    
                % CVTYPE == 100 IS THE BOOTSTRAP STD PLUG-IN ASY (PA) CRIT VALUE.  
          ZQUANT = ZQUANT;  % FOR NON-DATA-DEPENDENT CRIT VALUE, NO CHANGES 
                            % ARE MADE TO THE DATA ZQUANT.  

       elseif CVTYPE == 1||CVTYPE == 101; 
                % CVTYPE == 1 IS THE t TEST (PHI1) CRIT VALUE.    
                % CVTYPE == 101 IS THE BOOTSTRAP t TEST (PHI1) CRIT VALUE.  
          PSELECT = ( WC<=KAPPA*ones(P,1) );
                % MOMENT SELECT CRIT VALUE IS OBTAINED BY SIMULATING BASED
                % ON A VECTOR THAT INCLUDES THE J-TH ELEMENT if W_J <= KAPPA. */  
          if sum( PSELECT )==0;          
               ZQUANT = ZQUANT( :,P); 
               PSELECT(1) = 1;        
          else
               ZQUANT = ZQUANT(:,PSELECT == 1);
          end;
          V2 = V(PSELECT==1,PSELECT==1);   
                % HERE WE COMPUTE THE APPROPRIATE SUBMATRIX OF V THAT
                % CORRESPONDS TO THE SELECTED MOMENTS AND COMPUTE   
                % THE CORRELATION MATRIX THAT CORRESPONDS TO IT.  */
          STACK = sqrt(diag(V2));
          VC = V2 ./ (STACK * STACK');  
          VCINV = pinv( VC );

          V2 = VST(PSELECT==1,PSELECT==1);   
                % HERE WE COMPUTE THE APPROPRIATE SUBMATRIX OF VST THAT 
                % CORRESPONDS TO THE SELECTED MOMENTS AND COMPUTE    
                % THE CORRELATION MATRIX THAT CORRESPONDS TO IT. 
          STACK = sqrt(diag(V2));
          VCST = V2 ./ (STACK * STACK');  
          VCINVST = pinv( VCST );

       elseif CVTYPE == 11 || CVTYPE == 1011;   
          % CVTYPE == 11 IS THE t TEST CRIT VAL BASED ON AN AUTOMATIC 
          %              CHOICE OF KAPPA and ETA.
          % CVTYPE == 1011 IS THE BOOTSTRAP VERSION OF THIS. 
          % FIRST WE DETERMINE KAPPA AND ETA-AUTO BASED ON THE VALUE OF DELTA.   
          % THIS IS BASED ON A RULE THAT IS PROGRAMMED INTO THE PROCEDURE CALLED 
          % AUTOMAT, SEE AUTOMAT.M. 
          [KAPPA, ETAAUTO] = AUTOMAT(DELTA,P);
          ETA=ETA1+ETAAUTO;
                % THE KAPPA-AUTO CRITICAL VALUE USES THE ADDITIVE FACTOR ETAAUTO 
                % WHEN COMPUTING THE CRIT VAL.
          PSELECT = ( WC<=KAPPA*ones(P,1) );
                % MOMENT SELECT CRIT VALUE IS OBTAINED BY SIMULATING BASED
                % ON A VECTOR THAT INCLUDES THE J-TH ELEMENT if WC_J < KAPPA. 
          if sum( PSELECT )==0;           
                ZQUANT = ZQUANT( :,P);               
                PSELECT(1) = 1;                            
          else                                                 
               ZQUANT = ZQUANT(:,PSELECT == 1);  
          end;
          V2 = V(PSELECT==1 ,PSELECT==1);   
                % HERE WE COMPUTE THE APPROPRIATE SUBMATRIX OF V THAT CORRESPONDS
                % TO THE SELECTED MOMENTS AND COMPUTE THE CORRELATION MATRIX 
                % THAT CORRESPONDS TO IT.
          STACK = sqrt(diag(V2));
          VC = V2 ./ (STACK * STACK');  
          VCINV = pinv( VC );

          V2 = VST( PSELECT==1 ,PSELECT==1);   
                % HERE WE COMPUTE THE APPROPRIATE SUBMATRIX OF VST THAT
                % CORRESPONDS TO THE SELECTED MOMENTS AND COMPUTE THE
                % CORRELATION MATRIX THAT CORRESPONDS TO IT. 
          STACK = sqrt(diag(V2));
          VCST = V2 ./ (STACK * STACK');  
          VCINVST = pinv( VCST );
       end;                                                                      

       if BOOT == 1;    
          % WHEN BOOT == 1, WE ARE USING THE BOOTSTRAP AND WE WANT THE FINAL 
          % CORRELATION MATRIX USED TO BE BASED ON THE BOOTSTRAP CORRELATION
          % MATRIX ESTIMATOR. 
          VCFNL = VCST;
          VCINVFNL = VCINVST;
       elseif BOOT == 0;   
          % WHEN BOOT IS NOT 1, WE WANT THE FINAL CORRELATION MATRIX 
          % USED TO BE BASED ON THE CORRELATION MATRIX ESTIMATOR INPUTTED 
          % TO THE PROCEDURE. 
          VCFNL = VC;
          VCINVFNL = VCINV;
       end;
       LONGVEC( ((S-1)*R1+1):(S*R1) ) = STAT( ZQUANT, VCFNL, STATTYPE );
       S = S + 1;
    end;

    LONGVEC = sortrows( LONGVEC, 1 );
    QUANTIL = LONGVEC( ceil( (1-SIGLEVEL)*size(LONGVEC,1) ) ) + ETA;  
          % SIZE-CORRECTION IS CARRIED OUT BY USING THE NOMINAL SIGNIFICANCE 
          % LEVEL ALPHA and ADDING A CONSTANT ETA THAT REFINES THE GMS 
          % CRITICAL VALUE TO GIVE CORRECT ASY SIZE WHEN KAPPA DOES NOT DEPEND 
          % ON N. 
      